/*  execute this script in each company database to permit Misc Charge item types */

IF OBJECT_ID ( 'pcimportSales_Validate', 'P' ) IS NOT NULL   
    DROP PROCEDURE pcimportSales_Validate;  
GO

create procedure [dbo].[pcimportSales_Validate]  /* 2017-07-17 */
	@BatchID varchar(15),
	@UserID varchar(15),
	@InventoryRegistered tinyint,
	@FunctionalDecimalPlaces smallint,
	@NonIVQtyDecimalPlaces smallint,
	@NonIVCurrDecimalPlaces smallint,
	@Status varchar(80) out

/* with encryption */

as

set nocount on;

/* CONSTANTS */
declare @COSTCATEGORY_NONE smallint;					set @COSTCATEGORY_NONE = 1;
declare @COSTCATEGORY_LABOR smallint;					set @COSTCATEGORY_LABOR = 2;
declare @COSTCATEGORY_SUBCONTRACT smallint;				set @COSTCATEGORY_SUBCONTRACT = 3;
declare @COSTCATEGORY_PURCHASE smallint;				set @COSTCATEGORY_PURCHASE = 4;
declare @COSTCATEGORY_MATERIAL smallint;				set @COSTCATEGORY_MATERIAL = 5;
declare @COSTCATEGORYSTRING_NONE varchar(4);			set @COSTCATEGORYSTRING_NONE = 'None';
declare @COSTCATEGORYSTRING_LABOR varchar(5);			set @COSTCATEGORYSTRING_LABOR = 'Labor';
declare @COSTCATEGORYSTRING_SUBCONTRACT varchar(11);	set @COSTCATEGORYSTRING_SUBCONTRACT = 'SubContract';
declare @COSTCATEGORYSTRING_PURCHASE varchar(8);		set @COSTCATEGORYSTRING_PURCHASE = 'Purchase';
declare @COSTCATEGORYSTRING_MATERIAL varchar(8);		set @COSTCATEGORYSTRING_MATERIAL = 'Material';
declare @COSTCATEGORY_SALE smallint;					set @COSTCATEGORY_SALE = 8;
declare @DEFAULT_FROM_SYSTEM varchar(25);				set @DEFAULT_FROM_SYSTEM = '<default from system>';
declare @EMPTYDATE datetime;							set @EMPTYDATE = '1900.01.01 00:00:00.000';
declare @FALSE tinyint;									set @FALSE = 0;
declare @IMPORTTYPE_SALES smallint;						set @IMPORTTYPE_SALES = 4;
declare @IV_SALES_INVENTORY smallint;					set @IV_SALES_INVENTORY = 1;
declare @IV_DISCONTINUED smallint;						set @IV_DISCONTINUED = 2;
declare @IV_KIT smallint;								set @IV_KIT = 3;
declare @IV_MISC_CHARGES smallint;						set @IV_MISC_CHARGES = 4;
declare @IV_SERVICE smallint;							set @IV_SERVICE = 5;
declare @IV_FLAT_FEE smallint;							set @IV_FLAT_FEE = 6;
declare @STATUS_OK varchar(2);							set @STATUS_OK = 'ok';
declare @TRACK_NONE smallint;							set @TRACK_NONE = 1;
declare @TRACK_SERIALS smallint;						set @TRACK_SERIALS = 2;
declare @TRACK_LOTS smallint;							set @TRACK_LOTS = 3;
declare @TRUE tinyint;									set @TRUE = 1;
declare @TRXPROCESS_NOTSET smallint;					set @TRXPROCESS_NOTSET = 0;
declare @TRXPROCESS_WORK smallint;						set @TRXPROCESS_WORK = 1;
declare @TRXPROCESS_APPROVED smallint;					set @TRXPROCESS_APPROVED = 3;
declare @TRXPROCESS_COMPLETED smallint;					set @TRXPROCESS_COMPLETED = 4;
declare @TRXPROCESS_NOTREQUIRED smallint;				set @TRXPROCESS_NOTREQUIRED = 5;
declare @TRXSTATE_IMPORTWITHERRORS smallint;			set @TRXSTATE_IMPORTWITHERRORS = 0;
declare @TRXSTATE_IMPORTNOERRORS smallint;				set @TRXSTATE_IMPORTNOERRORS = 1;
declare @TRXSTATE_PROCESS smallint;						set @TRXSTATE_PROCESS = 2;


/* Locals */
declare @EnableMultiBin tinyint;
declare @ErrorMessage nvarchar(4000);
declare @ErrorNumber int;
declare @ErrorSeverity int;
declare @ErrorState int;
declare @Retry int;
declare @UseOverheadProcessing tinyint;


set @Status = @STATUS_OK;


/* Get info from PC System Setup */
select
	@UseOverheadProcessing = zUseOverheadProcessing
from
	pc00100
where
	zSetupKey = 1;

if @UseOverheadProcessing is null begin
	set @Status = 'PC System Setup record missing'
	return 0;
end;


select
	@EnableMultiBin = ENABLEMULTIBIN
from
	IV40100 as IV_SETP
where
	SETUPKEY = 1;


/*----+----+----+----+----+----+----+----+----+----+----*/
/* copy records to temp table to minimize deadlocks		*/
/*----+----+----+----+----+----+----+----+----+----+----*/
set @Retry = 30;
while @Retry > 0 begin

	begin try

		if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;

		select
			*
		into
			#ImportSalesValidate
		from
			pc10005
		where
			zBatchID = @BatchID
			and zImportTypeDDL = @IMPORTTYPE_SALES
			and zTransactionState in (@TRXSTATE_IMPORTNOERRORS, @TRXSTATE_PROCESS);

		set @Retry = 0;

	end try
	begin catch

		set @Retry = @Retry - 1;

		if @Retry <= 0 begin

			raiserror('Deadlock retries exhausted', 16, 1);

		end else if error_number() = 1205 begin

			waitfor delay '00:00:01';

		end else begin

			set @Retry = 0;
			select @ErrorMessage = 'Caught: Error %d, Level %d, State %d, Message: ' + error_message(), @ErrorNumber = error_number(), @ErrorSeverity = error_severity(), @ErrorState = error_state();
			raiserror(@ErrorMessage, 16, 1, @ErrorNumber, @ErrorSeverity, @ErrorState);

		end;

	end catch;

end;

alter table #ImportSalesValidate add primary key nonclustered ( zBatchID, zImportNumber, zImportRow ) on [PRIMARY];


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 1	Test for some common errors and set constant PC fields			*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* change transaction state to 2 */
	zTransactionState = @TRXSTATE_PROCESS,

	/* set zCostCategoryDDL */
	zCostCategoryDDL = case pcTransactionImport.zCostCategoryString
		when @COSTCATEGORYSTRING_NONE then @COSTCATEGORY_NONE
		when @COSTCATEGORYSTRING_MATERIAL then @COSTCATEGORY_MATERIAL
		else 0
	end,

	/* refresh string to correct case */
	zCostCategoryString = case pcTransactionImport.zCostCategoryString
		when @COSTCATEGORYSTRING_NONE then @COSTCATEGORYSTRING_NONE
		when @COSTCATEGORYSTRING_MATERIAL then @COSTCATEGORYSTRING_MATERIAL
		else zCostCategoryString
	end,

	/* set zQty */
	zQty = case pcTransactionImport.zCostCategoryString
		when @COSTCATEGORYSTRING_NONE then 0
		else pcTransactionImport.zBillingWorkQty
	end,

	/* set the user id */
	zUserID = @UserID,

	/* set StatusDDL's */
	zBillingStatusDDL = @TRXPROCESS_APPROVED,
	zExpenseStatusDDL = @TRXPROCESS_NOTREQUIRED,
	zPayrollStatusDDL = @TRXPROCESS_NOTREQUIRED,

	/* set Originating Date to imported date */
	zOrigDocumentDate = case
		when pcTransactionImport.zOrigDocumentDate = @EMPTYDATE then pcTransactionImport.zTransactionDate
		else pcTransactionImport.zOrigDocumentDate
	end,

	zPeriodNumber = coalesce(pcPeriodMaster.zPeriodNumber, 0),

	zDistHistoryYear = coalesce(SY_Period_HDR.YEAR1, 0),

	zUnitCost = 0,

	/* validate some common fields */
	zValidData = case
		when pcTransactionImport.zCostCategoryString not in (@COSTCATEGORYSTRING_NONE, @COSTCATEGORYSTRING_MATERIAL) then @FALSE
		when pcTransactionImport.zProjectID = '' then @FALSE
		when pcTransactionImport.zTaskID = '' then @FALSE
		when pcTransactionImport.zItemID = '' then @FALSE
		when pcTransactionImport.zUofM = '' then @FALSE
		when pcTransactionImport.zTransactionDate = @EMPTYDATE then @FALSE
		when pcTransactionImport.zBillingWorkQty = 0 then @FALSE
		when pcTransactionImport.zUnitPrice <= 0 then @FALSE
		when pcPeriodMaster.zPeriodNumber is null then @FALSE
		when SY_Period_HDR.YEAR1 is null then @FALSE
		else @TRUE
	end,

	zErrorDescription = case
		when pcTransactionImport.zCostCategoryString not in (@COSTCATEGORYSTRING_NONE, @COSTCATEGORYSTRING_MATERIAL) then 'Invalid Cost Category'
		when pcTransactionImport.zProjectID = '' then 'Missing Project ID'
		when pcTransactionImport.zTaskID = '' then 'Missing Task ID'
		when pcTransactionImport.zItemID = '' then 'Missing Item Number'
		when pcTransactionImport.zUofM = '' then 'Missing Unit Of Measure'
		when pcTransactionImport.zTransactionDate = @EMPTYDATE then 'Missing Transaction Date'
		when pcTransactionImport.zBillingWorkQty = 0 then 'Billing Quantity Required'
		when pcTransactionImport.zUnitPrice <= 0 then 'Invalid Unit Price'
		when pcPeriodMaster.zPeriodNumber is null then 'Unable to retrieve Period Number'
		when SY_Period_HDR.YEAR1 is null then 'Unable to retrieve Distribution History Year'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join pc00110 as pcPeriodMaster
		on pcTransactionImport.zTransactionDate between pcPeriodMaster.zDateBegin and pcPeriodMaster.zDateEnd
	left join SY40101 as SY_Period_HDR
		on pcTransactionImport.zTransactionDate between SY_Period_HDR.FSTFSCDY and SY_Period_HDR.LSTFSCDY
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState in (@TRXSTATE_IMPORTNOERRORS, @TRXSTATE_PROCESS);  -- This excludes import errors until the are updated by the user

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 1 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/*	STEP 2.	Validate zSubCostCategoryString and set zSubCostCategory.		*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh string from master table to correct case */
	zSubCostCategoryString = coalesce(pcCostCategoryMaster.zSubCostCategoryString, pcTransactionImport.zSubCostCategoryString),

	zSubCostCategory = coalesce(pcCostCategoryMaster.zSubCostCategory, 0),

	zValidData = case
		when pcCostCategoryMaster.zSubCostCategory is not null then @TRUE
		else @FALSE
	end,

	zErrorDescription = case
		when pcCostCategoryMaster.zSubCostCategory is not null then ''
		else 'Invalid Sub-Cost Category'
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join pc00150 as pcCostCategoryMaster
		on pcCostCategoryMaster.zCostCategoryDDL = pcTransactionImport.zCostCategoryDDL
		and pcCostCategoryMaster.zSubCostCategoryString = pcTransactionImport.zSubCostCategoryString
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 2 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/*	STEP 3.  Validate zDepartmentID if supplied.		REMOVED				*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/*	STEP 4.  Validate zProjectID and set fields based on Project			*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh string to correct case */
	zProjectID = coalesce(pcProjectMaster.zProjectID, pcTransactionImport.zProjectID),

	zOrigID = coalesce(RM_Customer_MSTR.CUSTNMBR, pcTransactionImport.zOrigID),

	zOrigName = coalesce(RM_Customer_MSTR.CUSTNAME, pcTransactionImport.zOrigName),

	zBurdenStatusDDL = coalesce(
		case
			when @UseOverheadProcessing = @TRUE
				and pcProjectMaster.zNoStdLaborPosting = @FALSE then @TRXPROCESS_WORK
			else @TRXPROCESS_NOTREQUIRED
		end,
		pcTransactionImport.zBurdenStatusDDL),

	zValidData = case
		when pcProjectMaster.zProjectID is null then @FALSE					-- Invalid ProjectID
		when pcProjectMaster.zProjectStatusDDL != 1 then @FALSE				-- Project not open
		when pcProjectMaster.zCustomerID = '' then @FALSE					-- Missing CustomerID
		when RM_Customer_MSTR.CUSTNMBR is null then @FALSE					-- Invalid CustomerID
		else @TRUE															-- Valid
	end,

	zErrorDescription = case
		when pcProjectMaster.zProjectID is null then 'Invalid Project ID'
		when pcProjectMaster.zProjectStatusDDL != 1 then 'Project status is not open'
		when pcProjectMaster.zCustomerID = '' then 'Customer ID not set in Project Master record'
		when RM_Customer_MSTR.CUSTNMBR is null then 'Invalid Customer ID'
		else ''		-- No Errors
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join pc00200 as pcProjectMaster
		on pcProjectMaster.zProjectID = pcTransactionImport.zProjectID
	left join RM00101 as RM_Customer_MSTR
		on RM_Customer_MSTR.CUSTNMBR = pcProjectMaster.zCustomerID
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 4 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 5.	Validate zTaskID and set fields based on Task					*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh string to correct case */
	zTaskID = coalesce(pcTaskMaster.zTaskID, pcTransactionImport.zTaskID),

	/* Validate against Task */
	zValidData = case
		when pcTaskMaster.zTaskID is null then @FALSE										-- Invalid TaskID
		when pcTaskMaster.zTaskStatusDDL != 1 then @FALSE									-- Task not open
		when pcTaskMaster.zAllowSlsCostCat = @FALSE
			and pcTaskMaster.zAllowScheduledBillings = @FALSE then @FALSE					-- Task doesn't allow sales
		when pcTransactionImport.zCostCategoryDDL = @COSTCATEGORY_MATERIAL
			and pcTaskMaster.zAllowMtrlCostCat = @FALSE then @FALSE							-- Material not allowed
		else @TRUE																			-- Valid ID
	end,

	/* set error description */
	zErrorDescription = case
		when pcTaskMaster.zTaskID is null then 'Invalid Task ID'
		when pcTaskMaster.zTaskStatusDDL != 1 then 'Task status is not open'
		when pcTaskMaster.zAllowSlsCostCat = @FALSE
			and pcTaskMaster.zAllowScheduledBillings = @FALSE then 'Sales transactions not allowed for this task'
		when pcTransactionImport.zCostCategoryDDL = @COSTCATEGORY_MATERIAL
			and pcTaskMaster.zAllowMtrlCostCat = @FALSE then 'Material transactions not allowed for this task'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join pc00300 as pcTaskMaster
		on pcTaskMaster.zProjectID = pcTransactionImport.zProjectID
		and pcTaskMaster.zTaskID = pcTransactionImport.zTaskID
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 5 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 6a	Validate zItemID when Cost Category = Material					*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh string to correct case */
	zItemID = coalesce(IV_Item_MSTR.ITEMNMBR, pcTransactionImport.zItemID),

	/* set item description if not imported */
	zItemDescription = case
		when pcTransactionImport.zItemDescription = '' then coalesce(IV_Item_MSTR.ITEMDESC, pcTransactionImport.zItemDescription)
		else pcTransactionImport.zItemDescription
	end,

	/* Update the Qty for Items */
	/* If item type Service then cost = 0 and qty = 0 and Unit cost = 0 */
	/* If item type Sales Inventory then qty = billing qty and calcualte extended cost. */
	zQty = case when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY
		then zQty
		else 0
	end,

	zAccountIndex = case when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY
		then zAccountIndex
		else 0
	end,

	zAccountNumberString = case when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY
		then zAccountNumberString
		else ''
	end,

	zUofM = coalesce(IV_UofM_SETP_DTL.UOFM, pcTransactionImport.zUofM),

	zUnitCost = case when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY
		then coalesce(round(IV_Item_MSTR.CURRCOST * IV_UofM_SETP_DTL.QTYBSUOM, IV_Item_MSTR.DECPLCUR - 1), 0)
		else 0
	end,

	zValidData = case
		when @InventoryRegistered = @FALSE then @FALSE
		when IV_Item_MSTR.ITEMNMBR is null then @FALSE
		when IV_Item_MSTR.INACTIVE = @TRUE then @FALSE
		when ivItemInactive.ITEMNMBR is not null then @FALSE
		when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY and @EnableMultiBin = @TRUE then @FALSE
		when IV_Item_MSTR.ITEMTYPE not in (@IV_SALES_INVENTORY, @IV_MISC_CHARGES, @IV_SERVICE) then @FALSE
		when IV_Item_MSTR.ITMTRKOP != @TRACK_NONE then @FALSE
		when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY and pcTransactionImport.zBillingWorkQty < 0 then @FALSE
		when IV_UofM_SETP_DTL.UOFM is null then @FALSE
		when pcTransactionImport.zBillingWorkQty != round(pcTransactionImport.zBillingWorkQty, IV_Item_MSTR.DECPLQTY - 1, 1) then @FALSE
		when pcTransactionImport.zUnitPrice != round(pcTransactionImport.zUnitPrice, IV_Item_MSTR.DECPLCUR - 1, 1) then @FALSE
		else @TRUE
	end,

	zErrorDescription = case
		when @InventoryRegistered = @FALSE then 'Inventory Module not registered'
		when IV_Item_MSTR.ITEMNMBR is null then 'Invalid Item Number'
		when IV_Item_MSTR.INACTIVE = @TRUE then 'Item is Inactive'
		when ivItemInactive.ITEMNMBR is not null then 'Item is Inactive'
		when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY and @EnableMultiBin = @TRUE then 'Multiple Bins not supported'
		when IV_Item_MSTR.ITEMTYPE not in (@IV_SALES_INVENTORY, @IV_MISC_CHARGES, @IV_SERVICE) then 'Unsupported Item Type'
		when IV_Item_MSTR.ITMTRKOP != @TRACK_NONE then 'Unsupported Item Tracking Type'
		when IV_Item_MSTR.ITEMTYPE = @IV_SALES_INVENTORY and pcTransactionImport.zBillingWorkQty < 0 then 'Sales Inventory Returns not supported'
		when IV_UofM_SETP_DTL.UOFM is null then 'Invalid Unit Of Measure'
		when pcTransactionImport.zBillingWorkQty != round(pcTransactionImport.zBillingWorkQty, IV_Item_MSTR.DECPLQTY - 1, 1) then 'Invalid Billing Quantity Decimal Places'
		when pcTransactionImport.zUnitPrice != round(pcTransactionImport.zUnitPrice, IV_Item_MSTR.DECPLCUR - 1, 1) then 'Invalid Unit Price Decimal Places'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join IV00101 as IV_Item_MSTR
		on IV_Item_MSTR.ITEMNMBR = pcTransactionImport.zItemID
	left join IV40201 as IV_UofM_SETP_HDR
		on IV_UofM_SETP_HDR.UOMSCHDL = IV_Item_MSTR.UOMSCHDL
	left join IV40202 as IV_UofM_SETP_DTL
		on IV_UofM_SETP_DTL.UOMSCHDL = IV_UofM_SETP_HDR.UOMSCHDL
		and IV_UofM_SETP_DTL.EQUIVUOM = IV_UofM_SETP_HDR.BASEUOFM
		and IV_UofM_SETP_DTL.UOFM = pcTransactionImport.zUofM
	left join IV00114 as ivItemInactive
		on ivItemInactive.ITEMNMBR = IV_Item_MSTR.ITEMNMBR
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zCostCategoryDDL = @COSTCATEGORY_MATERIAL;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 6a failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 6b	Validate zItemID when Cost Category is not Material				*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zItemDescription = case
		when pcTransactionImport.zItemDescription = '' then pcTransactionImport.zItemID
		else pcTransactionImport.zItemDescription
	end,

	zAccountIndex = 0,
	zAccountNumberString = '',

	zValidData = case
		when pcTransactionImport.zItemID = '' then @FALSE
		when IV_Item_MSTR.ITEMNMBR is not null then @FALSE
		when pcTransactionImport.zUofM = '' then @FALSE
		when pcTransactionImport.zBillingWorkQty != round(pcTransactionImport.zBillingWorkQty, @NonIVQtyDecimalPlaces, 1) then @FALSE
		when pcTransactionImport.zUnitPrice != round(pcTransactionImport.zUnitPrice, @NonIVCurrDecimalPlaces, 1) then @FALSE
		else @TRUE
	end,

	zErrorDescription = case
		when pcTransactionImport.zItemID = '' then 'Missing Item Number'
		when IV_Item_MSTR.ITEMNMBR is not null then 'Inventory Item not allowed when Cost Category is not Material'
		when pcTransactionImport.zUofM = '' then 'Missing Unit Of Measure'
		when pcTransactionImport.zBillingWorkQty != round(pcTransactionImport.zBillingWorkQty, @NonIVQtyDecimalPlaces, 1) then 'Invalid Billing Quantity Decimal Places'
		when pcTransactionImport.zUnitPrice != round(pcTransactionImport.zUnitPrice, @NonIVCurrDecimalPlaces, 1) then 'Invalid Unit Price Decimal Places'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join IV00101 as IV_Item_MSTR
		on IV_Item_MSTR.ITEMNMBR = pcTransactionImport.zItemID
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zCostCategoryDDL != @COSTCATEGORY_MATERIAL;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 6b failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 7	Set Cost & Billing fields										*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zCostAmt = round(zQty * zUnitCost, @FunctionalDecimalPlaces),
	zBillingWorkAmt = round(zBillingWorkQty * zUnitPrice, @FunctionalDecimalPlaces)
where
	zBatchID = @BatchID
	and zImportTypeDDL = @IMPORTTYPE_SALES
	and zTransactionState = @TRXSTATE_PROCESS
	and zValidData = @TRUE;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 7 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* Account strings should contain either a valid account string or			*/
/* <default from system>. Anything else, including empty string, is invalid	*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/

/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 8a	Clear the account index and string if the cost category is None	*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zAccountNumberString = '',
	zAccountIndex = 0
where
	zBatchID = @BatchID
	and zImportTypeDDL = @IMPORTTYPE_SALES
	and zTransactionState = @TRXSTATE_PROCESS
	and zValidData = @TRUE
	and zCostCategoryDDL = @COSTCATEGORY_NONE;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 8b	Get the cost account index from supplied account string			*/
/*			when not CostCategory None and not <default from system>		*/
/*			If unit cost is 0 then an empty cost acct is ok					*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh account string to correct case */
	zAccountNumberString = coalesce(GL_Account_Index_MSTR.ACTNUMST, pcTransactionImport.zAccountNumberString),

	zAccountIndex = coalesce(GL_Account_Index_MSTR.ACTINDX, 0),

	zValidData = case
		when pcTransactionImport.zAccountNumberString = '' and pcTransactionImport.zUnitCost != 0 then @FALSE
		when pcTransactionImport.zAccountNumberString != '' and GL_Account_Index_MSTR.ACTINDX is null then @FALSE
		else @TRUE
	end,

	zErrorDescription = case
		when pcTransactionImport.zAccountNumberString = '' and pcTransactionImport.zUnitCost != 0 then 'Cost Account Number Required'
		when pcTransactionImport.zAccountNumberString != '' and GL_Account_Index_MSTR.ACTINDX is null then 'Invalid Cost Account Number'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join GL00105 as GL_Account_Index_MSTR
		on GL_Account_Index_MSTR.ACTNUMST = pcTransactionImport.zAccountNumberString
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zCostCategoryDDL != @COSTCATEGORY_NONE
	and pcTransactionImport.zAccountNumberString != @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 8a failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 8c1	Get the cost account index if <default from system>				*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zAccountIndex = dbo.opcGetAccountIndex_F(
		zCostCategoryDDL,
		zProjectID,
		zTaskID,
		'',
		0)
where
	zBatchID = @BatchID
	and zImportTypeDDL = @IMPORTTYPE_SALES
	and zTransactionState = @TRXSTATE_PROCESS
	and zValidData = @TRUE
	and zCostCategoryDDL != @COSTCATEGORY_NONE
	and zAccountNumberString = @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 8b1 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 8c2	Get the cost account string if <default from system>			*/
/*			If unit cost is 0 then an empty cost acct is ok					*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zAccountNumberString = coalesce(GL_Account_Index_MSTR.ACTNUMST, pcTransactionImport.zAccountNumberString),

	/* clear account index if failed */
	zAccountIndex = coalesce(GL_Account_Index_MSTR.ACTINDX, 0),

	/* the only way this could fail is if step 9b1 returned 0 for index */
	zValidData = case
		when GL_Account_Index_MSTR.ACTNUMST is not null then @TRUE
		when GL_Account_Index_MSTR.ACTNUMST is null and pcTransactionImport.zUnitCost = 0 then @TRUE
		else @FALSE
	end,

	zErrorDescription = case
		when GL_Account_Index_MSTR.ACTNUMST is not null then ''
		when GL_Account_Index_MSTR.ACTNUMST is null and pcTransactionImport.zUnitCost = 0 then ''
		else 'Unable to determine Cost Account from Project System'
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join GL00105 as GL_Account_Index_MSTR
		on GL_Account_Index_MSTR.ACTINDX = pcTransactionImport.zAccountIndex
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zCostCategoryDDL != @COSTCATEGORY_NONE
	and pcTransactionImport.zAccountNumberString = @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 8b2 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 9a	Get the sales account index from supplied account string		*/
/*			(not <default from system>)										*/
/*			If billing status is not required, then empty sales acct is ok	*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	/* refresh account string to correct case */
	zSalesAccountString = coalesce(GL_Account_Index_MSTR.ACTNUMST, pcTransactionImport.zSalesAccountString),

	zSalesAccountIndex = coalesce(GL_Account_Index_MSTR.ACTINDX, 0),

	zValidData = case
		when pcTransactionImport.zSalesAccountString = '' and pcTransactionImport.zBillingStatusDDL != @TRXPROCESS_NOTREQUIRED then @FALSE
		when pcTransactionImport.zSalesAccountString != '' and GL_Account_Index_MSTR.ACTINDX is null then @FALSE
		else @TRUE
	end,

	zErrorDescription = case
		when pcTransactionImport.zSalesAccountString = '' and pcTransactionImport.zBillingStatusDDL != @TRXPROCESS_NOTREQUIRED then 'Sales Account Number required'
		when pcTransactionImport.zSalesAccountString != '' and GL_Account_Index_MSTR.ACTINDX is null then 'Invalid Sales Account Number'
		else ''
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join GL00105 as GL_Account_Index_MSTR
		on GL_Account_Index_MSTR.ACTNUMST = pcTransactionImport.zSalesAccountString
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zSalesAccountString != @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 9a failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 9b1	Get the sales account index if <default from system>			*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zSalesAccountIndex = dbo.opcGetAccountIndex_F(
		@COSTCATEGORY_SALE,	-- Cost Category
		zProjectID, 		-- Project ID
		zTaskID,			-- Task ID
		'',					-- Vendor ID
		0)					-- Account Index
where
	zBatchID = @BatchID
	and zImportTypeDDL = @IMPORTTYPE_SALES
	and zTransactionState = @TRXSTATE_PROCESS
	and zValidData = @TRUE
	and zSalesAccountString = @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 9b1 failed with error code ' + ltrim(@@error);
	return 0;
end;


/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
/* STEP 9b2	Get the sales account string if <default from system>			*/
/*				If billing is not required, then empty sales acct is ok		*/
/*----+----+----+----+----+----+----+----+----+----+----+----+----+----+----*/
update
	#ImportSalesValidate
set
	zSalesAccountString = coalesce(GL_Account_Index_MSTR.ACTNUMST, pcTransactionImport.zSalesAccountString),

	/* clear sales account index if failed */
	zSalesAccountIndex = coalesce(GL_Account_Index_MSTR.ACTINDX, 0),

	/* the only way this could fail is if step 10b1 returned 0 for index */
	zValidData = case
		when GL_Account_Index_MSTR.ACTNUMST is not null then @TRUE
		when GL_Account_Index_MSTR.ACTNUMST is null and pcTransactionImport.zBillingStatusDDL = @TRXPROCESS_NOTREQUIRED then @TRUE
		else @FALSE
	end,

	zErrorDescription = case
		when GL_Account_Index_MSTR.ACTNUMST is not null then ''
		when GL_Account_Index_MSTR.ACTNUMST is null and pcTransactionImport.zBillingStatusDDL = @TRXPROCESS_NOTREQUIRED then ''
		else 'Unable to determine Sales Account from Project System'
	end
from
	#ImportSalesValidate as pcTransactionImport
	left join GL00105 as GL_Account_Index_MSTR
		on GL_Account_Index_MSTR.ACTINDX = pcTransactionImport.zSalesAccountIndex
where
	pcTransactionImport.zBatchID = @BatchID
	and pcTransactionImport.zImportTypeDDL = @IMPORTTYPE_SALES
	and pcTransactionImport.zTransactionState = @TRXSTATE_PROCESS
	and pcTransactionImport.zValidData = @TRUE
	and pcTransactionImport.zSalesAccountString = @DEFAULT_FROM_SYSTEM;

if @@error != 0 begin
	if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;
	set @Status = 'pcimportSales_Validate Step 9b2 failed with error code ' + ltrim(@@error);
	return 0;
end;


set @Retry = 30;
while @Retry > 0 begin

	begin transaction;

	begin try
		update
			pc10005
		set
			pc10005.zAccountIndex = Import.zAccountIndex,
			pc10005.zAccountNumberString = Import.zAccountNumberString,
			pc10005.zBillingStatusDDL = Import.zBillingStatusDDL,
			pc10005.zBillingWorkAmt = Import.zBillingWorkAmt,
			pc10005.zBurdenStatusDDL = Import.zBurdenStatusDDL,
			pc10005.zCostAmt = Import.zCostAmt,
			pc10005.zCostCategoryDDL = Import.zCostCategoryDDL,
			pc10005.zCostCategoryString = Import.zCostCategoryString,
			pc10005.zDepartmentID = Import.zDepartmentID,
			pc10005.zDistHistoryYear = Import.zDistHistoryYear,
			pc10005.zErrorDescription = Import.zErrorDescription,
			pc10005.zExpenseStatusDDL = Import.zExpenseStatusDDL,
			pc10005.zItemDescription = Import.zItemDescription,
			pc10005.zItemID = Import.zItemID,
			pc10005.zOrigDocumentDate = Import.zOrigDocumentDate,
			pc10005.zOrigID = Import.zOrigID,
			pc10005.zOrigName = Import.zOrigName,
			pc10005.zPayrollStatusDDL = Import.zPayrollStatusDDL,
			pc10005.zPeriodNumber = Import.zPeriodNumber,
			pc10005.zProjectID = Import.zProjectID,
			pc10005.zQty = Import.zQty,
			pc10005.zSalesAccountIndex = Import.zSalesAccountIndex,
			pc10005.zSalesAccountString = Import.zSalesAccountString,
			pc10005.zSubCostCategory = Import.zSubCostCategory,
			pc10005.zSubCostCategoryString = Import.zSubCostCategoryString,
			pc10005.zTaskID = Import.zTaskID,
			pc10005.zTransactionState = Import.zTransactionState,
			pc10005.zUnitCost = Import.zUnitCost,
			pc10005.zUofM = Import.zUofM,
			pc10005.zUserID = Import.zUserID,
			pc10005.zValidData = Import.zValidData
		from
			pc10005
			join #ImportSalesValidate as Import
				on Import.zBatchID = pc10005.zBatchID
				and Import.zImportNumber = pc10005.zImportNumber
				and Import.zImportRow = pc10005.zImportRow;

		commit transaction;
		set @Retry = 0;

	end try
	begin catch

		set @Retry = @Retry - 1;
		if xact_state() != 0 rollback transaction;

		if @Retry <= 0 begin

			raiserror('Deadlock retries exhausted', 16, 2);

		end else if error_number() = 1205 begin

			waitfor delay '00:00:01';

		end else begin

			set @Retry = 0;
			select @ErrorMessage = 'Caught: Error %d, Level %d, State %d, Message: ' + error_message(), @ErrorNumber = error_number(), @ErrorSeverity = error_severity(), @ErrorState = error_state();
			raiserror(@ErrorMessage, 16, 2, @ErrorNumber, @ErrorSeverity, @ErrorState);

		end;

	end catch;

end;

if object_id('tempdb..#ImportSalesValidate', 'U') is not null drop table #ImportSalesValidate;

return 0;
go

-- this will grant select rights to the Procedure.  
grant All on pcimportSales_Validate to DYNGRP
Go